@@ -2,11 +2,14 @@
 
 import os
 import rsa
+import string
+import secrets
 import hashlib as hash
 from Crypto.Cipher import AES
 from Crypto.Cipher import Blowfish
 from Crypto.Protocol.KDF import PBKDF2
-from random import randbytes
+from cryptography.fernet import Fernet
+from random import randbytes, shuffle
 
 
 def hash_(StringToHash=str, hash_code = str, return_type = "hex") -> str:
@@ -49,7 +52,58 @@ def hash_(StringToHash=str, hash_code = str, return_type = "hex") -> str:
             return hash_obj.hexdigest()
 
 
-def sec_delete(self, file_path, random_fill = True, null_fill = True, passes = 35) -> int:
+def random_choice(list):
+    chosen = secrets.choice(list)
+    return chosen
+
+
+def compare_hashes(hash_1=str, hash_2=str):
+    """
+    hash comparision function. 
+
+    Takes 2 strings and compares them to see if they are the same.
+    returns a boolean value in such a way to reduce timing attack efficacy.
+    """
+    result = secrets.compare_digest(hash_1, hash_2)
+    return result
+
+
+def token_generation(size=int, return_type = str):
+    """
+    Simplifed method for interfacing with the secrets module.
+
+    return_type: What is being returned. modes are 'URL', 'HEX', and 'BYTES'
+    
+    size: the number of bytes in the token to be generated.
+    """
+    if return_type == "HEX":
+        token = secrets.token_hex(size)
+        return token
+    if return_type == "BYTES":
+        token = secrets.token_bytes(size)
+        return token
+    if return_type == "URL":
+        token = secrets.token_urlsafe(size)
+        return token
+
+
+def gen_random_password(length=int):
+    characters = list(string.ascii_letters + string.digits + "!@#$%^&*()")
+    shuffle(characters)
+    password = []
+    for i in range(length):
+        password.append(secrets.choice(characters))
+    shuffle(password)
+    final_password = "".join(password)
+    return final_password
+
+
+def sec_delete(file_path, random_fill = True, null_fill = True, passes = 35) -> int:
+    """
+    Secure file deletion function with overwriting and null filling.
+
+    It is best practice to combine this with another secure file deletion protocol.
+    """
     if "/home/" not in file_path or "/." in file_path:
         return 1
     else:
@@ -67,7 +121,10 @@ def sec_delete(self, file_path, random_fill = True, null_fill = True, passes = 3
         return 0
 
 
-def delete(self, path):
+def delete(path):
+    """
+    File deletion function.
+    """
     if '/home/' not in path or "/." in path:
         return 1
     else:
@@ -91,7 +148,7 @@ class Ciphers:
         self.marker  = b"E1m%nj2i$bhilj"
 
 
-    # misc functions
+    # misc methods
 
 
     def generate_symmetric_key(self, salt) -> 'tuple[bytes, bytes]':
@@ -105,12 +162,17 @@ class Ciphers:
             return key
 
 
+    def generate_FerNet_key():
+        key = Fernet.generate_key()
+        return key
+
+
     def check_path_validity(self, path=str or None, decrypting=bool, type_=str) -> int:
         """
         A guard clause for file operations.
 
         Return codes:
-            0: File path is good.
+            0: File path is suitable.
             1: File path is None.
             2: File path is invalid.
             3: App does not have permission to access file.
@@ -162,7 +224,7 @@ class Ciphers:
             self.password = new_password
 
 
-    # RSA functions 
+    # RSA methods 
 
 
     def generate_RSA_keys(self):
@@ -183,10 +245,10 @@ class Ciphers:
         return decrypted_string
 
 
-        # AES functions
+        # AES methods
 
 
-    def AESencrypt_file(self, path=str) -> int:
+    def AES_encrypt_file(self, path=str) -> int:
         """
         A method for encrypting a file/folder object.
 
@@ -257,7 +319,7 @@ class Ciphers:
             return 0
 
 
-    def AESdecrypt_file(self, path=str) -> int:
+    def AES_decrypt_file(self, path=str) -> int:
         """
         A method for decrypting an encrypted file.
 
@@ -331,11 +393,18 @@ class Ciphers:
             return 0
         
 
-
-    # Blowfish functions
+    def AES_encrypt_string():
+        pass
 
 
-    def BLOencrypt_file(self, path=str):
+    def AES_decrypt_string():
+        pass
+
+
+    # Blowfish methods
+
+
+    def BLO_encrypt_file(self, path=str):
         """
         A method for encrypting a file using the Blowfish encryption algorithm.
         """
@@ -396,7 +465,7 @@ class Ciphers:
                         file.write(ciphered_data)
 
 
-    def BLOdecrypt_file(self, path=str):
+    def BLO_decrypt_file(self, path=str):
         """
         method for decrypting a file encrypted with the Blowfish encryption alogrithm
         """
@@ -457,3 +526,111 @@ class Ciphers:
                         file.write(original_data)
 
 
+    def BLO_encrypt_string(self, key=bytes or None, string_=str):
+        """
+        Method for encrypting strings using the Blowfish encryption alogrithm.
+        returns a tuple of (ciphered_string, iv, salt)
+        """
+        if key is None:
+            salt = os.urandom(32)
+            key = self.generate_symmetric_key(salt)
+        iv = os.urandom(8)
+        cipher = Blowfish.new(key, Blowfish.MODE_CFB, iv=iv)
+        ciphered_string = cipher.encrypt(string_)
+        return (ciphered_string, iv, key)
+
+
+    def BLO_decrypt_string(self, encrypted_string=str, key=bytes, iv=bytes):
+        """
+        Method for encrypting strings using the Blowfish encryption alogrithm.
+        returns a tuple of (ciphered_string, iv, salt)
+        """
+        if key is None:
+            return 1
+        cipher = Blowfish.new(key, Blowfish.MODE_CFB, iv=iv)
+        decrypted_string = cipher.decrypt(encrypted_string)
+        return (decrypted_string)
+
+    # FerNet methods
+
+
+    def FerNet_encrypt_file(self, path=str, key = str or None):
+        if os.path.isfile(path) is True:
+            return_code = self.check_path_validity(path, decrypting=False, type_="FER")
+            if return_code != 0:
+                return return_code
+            if key is None:
+                key = self.generate_FerNet_key()
+            f = Fernet(key)
+            with open(path, "rb") as file:
+                data = file.read()
+            encrypted_data = f.encrypt(data)
+            with open (path, "wb") as file:
+                file.write(b"FER")
+                file.write(self.marker)
+                file.write(encrypted_data)
+            return 0
+        if os.path.isdir(path) is True:
+            for root, dirs, files in os.walk(path):
+                for file in files:
+                    file_path = os.path.join(root, file)
+                    return_code = self.check_path_validity(file_path, decrypting=False, type_="FER")
+                    if key is None:
+                        key = self.generate_FerNet_key()
+                    f = Fernet(key)
+                    with open (file_path, "rb") as file:
+                        data = file.read()
+                    encrypted_data = f.encrypt(data)
+                    with open(path, "wb") as file:
+                        file.write(b"FER")
+                        file.write(self.marker)
+                        file.write(encrypted_data)
+            return 0
+    
+
+    def FerNet_decrypt_file(self, path=str, key=bytes):
+        if os.path.isfile(path) is True:
+            return_code = self.check_path_validity(path, decrypting=True, type_="FER")
+            if return_code != 0:
+                return return_code
+            f = Fernet(key)
+            with open(path, "rb") as file:
+                file.read(3)
+                file.read(14)
+                ciphered_data = file.read()
+            decrypted_data = f.decrypt(ciphered_data)
+            with open(path, 'wb') as file:
+                file.write(decrypted_data)
+            return 0
+        if os.path.isdir(path) is True:
+            for root, dirs, files in os.walk(path):
+                for file in files:
+                    file_path = os.path.join(root, file)
+                    return_code = self.check_path_validity(file_path, decrypting=True, type_="FER")
+                    if return_code != 0:
+                        return return_code
+                    f = Fernet(key)
+                    with open(file_path, "rb") as file:
+                        file.read(3)
+                        file.read(14)
+                        ciphered_data = file.read()
+                    decrypted_data = f.decrypt(ciphered_data)
+                    with open(file_path, 'wb') as file:
+                        file.write(decrypted_data)
+            return 0
+    
+
+    def FerNet_encrypt_string(self, key = str or None, bstring=bytes):
+        if key is None:
+            key = self.generate_FerNet_key()
+        f = Fernet(key)
+        encrypted_text = f.encrypt(bstring)
+        return encrypted_text
+
+
+    def FerNet_decrypt_string(self, key=bytes, encrypted_string=bytes):
+        if key is None:
+            return 1
+        f = Fernet(key)
+        decrypted_text = f.encrypt(encrypted_string)
+        return decrypted_text
