@@ -1,6 +1,6 @@
 from setuptools import setup, find_packages
 
-VERSION = "0.2.5"
+VERSION = "0.3.1"
 DESCRIPTION = "A module for encryption and information security operations."
 setup(
     name= "better cryptography",
@@ -11,7 +11,7 @@ setup(
     long_description = open("/home/eternal_blue/better_cryptography/README.md").read(),
     long_description_content_type="text/markdown",
     packages = find_packages(),
-    install_requires = ['pycryptodome', "rsa"],
+    install_requires = ['pycryptodome', "rsa", "cryptography", "cffi"],
     keywords=["python", "encryption", "AES", "information security"],
     classifiers= [
         "Development Status :: 3 - Alpha",
